/**************************************/
/* H8/3048FEEPROM񤭹ߥץ */
/* Copyleft,1999,Yukio Mituiwa        */
/* Copyleft,2008,Shinichi Fuchita     */
/**************************************/

#include <h8/3048.h>
#define MAXADDR	0x1efff
#include <common.h>


#define WDTE		0xa57f
#define WDTD		0xa500
#define H8CLOCK		16
#define CONST_A		(H8CLOCK*25)/10+1
#define CONST_B		(H8CLOCK*7)/10+1
#define CONST_G		(H8CLOCK*9)/10+1

void waitloop(unsigned short i);


int _writebyte(unsigned long addr, unsigned char c)
{
	unsigned short delay;
	int i, m;

	unsigned char *dst = (unsigned char *) addr;
	if (*dst == c) return 0;

	FLMCR |= 0x40;				// VppE = 1
	waitloop(CONST_G);
	EBR1 = 1 << (addr >> 14);
	*dst = c;
	delay = CONST_A;
	for (i = 0; i < 6; i++) {
		*(unsigned short *) &TCSR = WDTE;
		FLMCR |= 0x01;			// P = 1
		waitloop(delay);
		FLMCR &= ~0x01;			// P = 0
		*(unsigned short *) &TCSR = WDTD;
		FLMCR |= 0x04;			// VP = 1
		waitloop(CONST_B);
		m = (*dst != c);
		FLMCR &= ~0x04;			// VP = 0
		if (!m) break;
		delay <<= 1;
	}
	EBR1 = 0;
	FLMCR &= ~0x40;				// VppE = 0
	return m ? -1 : 0;
}

/* ꥢݡȤ1Хɤ߹ */
int uart_getbyte(void)
{
	unsigned char c;
	while (!(SSR1 & 0x40));
	c = RDR1;
	SSR1 &= ~0x40;
	return (int) c;
}

/* ꥢݡȤ1ХȽ񤭹 */
void uart_putbyte(int c)
{
	while (!(SSR1 & 0x80));
	TDR1 = c;
	SSR1 &= ~0x80;
}

int main(void)
{
	unsigned char buffer[256];
	unsigned long addr, value;
	int c, i, j, n, len;

	SMR1 = 0;
	SCR1 = 0x30;
	SSR1 &= 0x87;

	uart_putbyte(BRR1);
	uart_putbyte(~BRR1);

	len = 0;
	addr = value = 0;
	while (1) {
		c = uart_getbyte();
		if (c >= 0) {
			if (len > 0) {
				buffer[256 - len] = c;
				len--;
			} else if (c < 0x80) {
				value = (value << 7) | c;
			} else {
				switch (c) {
				case OP_SETBINARY256:
					len = 256;
					break;
				case OP_SETADDRESS:
					addr = value;
					break;
				case OP_CHECKSUM:
					for (i = c = 0; i < 256; i++) {
						c += buffer[i];
					}
					uart_putbyte(c);
					break;
				case OP_PROGRAM:
					for (i = 0; i < 256; i++) {
						_writebyte(addr + i, buffer[i]);
						uart_putbyte(*(unsigned char *)(addr + i));
					}
					break;
				}
			}
		}
	}
}
