/**************************************/
/* H8/3067FEEPROM񤭹ߥץ */
/* Copyleft,2001,Yukio Mituiwa        */
/* Copyleft,2007,Shinichi Fuchita     */
/**************************************/

//#include "reg3067.h"
#include <h8/3067.h>
#include <common.h>

void waitloop(unsigned short n);

unsigned short delay1024us;
unsigned short delay1us;
unsigned short delay2us;
unsigned short delay4us;
unsigned short delay8us;
unsigned short delay16us;
unsigned short delay32us;
unsigned short delay64us;
unsigned short delay128us;
unsigned short delay200us;
unsigned short delay500us;

#define wait_1us() waitloop(delay1us)
#define wait_2us() waitloop(delay2us)
#define wait_4us() waitloop(delay4us)
#define wait_8us() waitloop(delay8us)
#define wait_16us() waitloop(delay16us)
#define wait_32us() waitloop(delay32us)
#define wait_64us() waitloop(delay64us)
#define wait_128us() waitloop(delay128us)


void write_pulse(int delay)
{
	FLMCR |= 0x10;				/* PSU <- 1                 */
	wait_64us();				/* wait 50[uS]              */
	FLMCR |= 0x01;				/* P <- 1, programing start         */
	waitloop(delay);			/* wait                 */
	FLMCR &= ~0x01;			/* P <- 0, programing stop      */
	wait_16us();
	FLMCR &= ~0x10;			/* PSU <- 0                 */
	wait_16us();
}

int write_block(unsigned char *dst, char *src)
{
	unsigned char tmp[32];
	int i, n, m;

	for (i = 0; i < 32; i++) {
		tmp[i] = 0xff;
	}

	FLMCR |= 0x40;				/* SWE <- 1 */
	wait_16us();				/* wait 16[uS] */

	for (n = 0; n < 100; n++) {
		m = 0;
		for (i = 0; i < 32; i++) {
			dst[i] = ~tmp[i] | src[i];
		}
		write_pulse(n < 6 ? delay200us : delay500us);
		FLMCR |= 0x04;						/* PV <- 1 */
		wait_4us();							/* wait 4[uS] */
		for (i = 0; i < 32; i++) {
			dst[i] = 0xff;					/* dummy write */
			wait_2us();							/* wait 2[uS] */
			tmp[i] = dst[i];
			if (tmp[i] != src[i]) {
				m = 1;						/* compare byte */
			}
		}
		FLMCR &= ~0x04;					/* PV <- 0 */
		wait_4us();         				/* wait 2[uS] */
		if (!m) {
			break;
		}
	}

	FLMCR &= ~0x40;			/* SWE <- 0 */
	wait_128us();			/* wait 100[uS] */

	return m ? -1 : 0;
}


int uart_getbyte()
{
	unsigned char c;
	while (!(SSR1 & 0x78));
	if (SSR1 & 0x40) {
		c = RDR1;
		SSR1 &= ~0x40;
		return c;
	}
	SSR1 = 0xc7;
	return -1;
}
void uart_putbyte(unsigned char c)
{
	while (!(SSR1 & 0x80));
	TDR1 = c;
	SSR1 &= ~0x80;
}

int read(char *buf, int size)
{
	int i;
	for (i = 0; i < size; i++) {
		int c = uart_getbyte();
		if (c < 0) break;
		buf[i] = c;
	}
	return i;
}

void write(char *buf, int size)
{
	int i;
	for (i = 0; i < size; i++) {
		uart_putbyte(buf[i]);
	}
}

extern unsigned long measure();


int main(void)
{
	unsigned char buffer[256];
	unsigned long addr, value;
	unsigned long n;
	int c, i, len;

	SMR1 = 0;
	SCR1 = 0x30;
	SSR1 &= 0x87;

	uart_putbyte(BRR1);
	uart_putbyte(~BRR1);

	delay1024us = 20000000 / 1000 / 12;
	delay1us = delay1024us >> 10;
	delay2us = delay1024us >> 9;
	delay4us = delay1024us >> 8;
	delay8us = delay1024us >> 7;
	delay16us = delay1024us >> 6;
	delay32us = delay1024us >> 5;
	delay64us = delay1024us >> 4;
	delay128us = delay1024us >> 3;
	delay200us = delay1024us * 200L / 1024;
	delay500us = delay1024us * 500L / 1024;

	len = 0;
	addr = value = 0;

	while (1) {
		c = uart_getbyte();
		if (c >= 0) {
			if (len > 0) {
				buffer[256 - len] = c;
				len--;
			} else if (c < 0x80) {
				value = (value << 7) | c;
			} else {
				switch (c) {
				case OP_SETBINARY256:
					len = 256;
					break;
				case OP_SETADDRESS:
					addr = value;
					break;
				case OP_CHECKSUM:
					break;
				case OP_PROGRAM:
					for (i = 0; i < 256; i += 32) {
						n = write_block((unsigned char *)addr + i, buffer + i);	/* EEPROM˥ǡ񤭹 */
						write((unsigned char *)addr + i, 32);
					}
					break;
				}
			}
		}
	}
}
