/**************************************/
/* H8/3664FEEPROM񤭹ߥץ */
/* Copyleft,2000,Yukio Mituiwa        */
/* Copyleft,2007,Shinichi Fuchita     */
/**************************************/

#include <h8/3664.h>
#include <common.h>

unsigned short delay1024us;
unsigned short delay1us;
unsigned short delay2us;
unsigned short delay4us;
unsigned short delay8us;
unsigned short delay16us;
unsigned short delay32us;
unsigned short delay64us;
unsigned short delay128us;
unsigned short delay200us;


void waitloop(unsigned short i);


//#define	wait_micro(mi_sec) for(wait = 0;wait < mi_sec;wait++)

#define wait_1us() waitloop(delay1us)
#define wait_2us() waitloop(delay2us)
#define wait_4us() waitloop(delay4us)
#define wait_8us() waitloop(delay8us)
#define wait_16us() waitloop(delay16us)
#define wait_32us() waitloop(delay32us)
#define wait_64us() waitloop(delay64us)
#define wait_128us() waitloop(delay128us)


inline static void write_pulse(int delay)
{
//	TCSRWD = 0x5a;				/* 01011010  ----  TCWD and TMWD is enabled */
//	TMWD = 0xff;				/* 11111111  ----  clock x 8192     */
//	TCWD = 255 - 13;			/* 6.6 [mS]                 */
//	TCSRWD = 0xa6;				/* 10100110  ----  WDT is start     */
	FLMCR1 |= 0x10;				/* PSU <- 1                 */
	wait_64us();				/* wait 50[uS]              */
	FLMCR1 |= 0x01;				/* P <- 1, programing start         */
	waitloop(delay);			/* wait                 */
	FLMCR1 &= ~0x01;			/* P <- 0, programing stop      */
	wait_8us();
	FLMCR1 &= ~0x10;			/* PSU <- 0                 */
	wait_8us();
//	TCSRWD = 0xa2;				/* 10100010  ----  WDT is stop      */
//	TCSRWD = 0xaa;				/* 10101010  ----  WDT is disabled  */
}

int write_block(char *dst, char *src)
{
	unsigned char tmp[128];
	int i, n, m;

	for (i = 0; i < 128; i++) {
		tmp[i] = dst[i];
	}

	FLMCR1 |= 0x40;				/* SWE <- 1 */
	wait_1us();				/* wait 1[uS] */

	for (n = 0; n < 100; n++) {
		m = 0;
		for (i = 0; i < 128; i++) {
			dst[i] = ~tmp[i] | src[i];
		}
		write_pulse(n < 6 ? delay32us : delay200us);
		FLMCR1 |= 0x04;						/* PV <- 1 */
		wait_4us();							/* wait 4[uS] */
		for (i = 0; i < 128; i++) {
			dst[i] = 0xff;					/* dummy write */
			wait_2us();							/* wait 2[uS] */
			tmp[i] = dst[i];
			if (tmp[i] != src[i])
				m = 1;						/* compare byte */
		}
		FLMCR1 &= ~0x04;					/* PV <- 0 */
		wait_2us();         				/* wait 2[uS] */
		if (!m) {
			for (i = 0; i < 128; i++) {
				dst[i] = src[i];
			}
			write_pulse(delay8us);
			break;
		}
	}

	FLMCR1 &= ~0x40;			/* SWE <- 0 */
	wait_128us();			/* wait 100[uS] */

	return m ? -1 : 0;
}


int uart_getbyte()
{
	while (!(SSR & 0x40)) {
		if (SSR & 0x38) {
			SSR = 0x40;
			return -1;
		}
	}
	return RDR;
}
void uart_putbyte(unsigned char c)
{
	while (!(SSR & 0x80));
	TDR = c;
}

int read(char *buf, int size)
{
	int i, c, d;

	for (i = 0; i < size; i++) {
		buf[i] = uart_getbyte();
	}
}

/* ꥢݡȤؽ񤭹 */
void write(char *buf, int size)
{
	int i;

	for (i = 0; i < size; i++) {
		uart_putbyte(buf[i]);
	}
}

int main(void)
{
	char buffer[256];
	int c, i, j, n, len;
	unsigned long addr, value;

	SMR = 0;
	SCR3 = 0x30;
	SSR &= 0x87;

	uart_putbyte(BRR);
	uart_putbyte(~BRR);

	delay1024us = 16000000 / 1000 / 12;
	delay1us = delay1024us >> 10;
	delay2us = delay1024us >> 9;
	delay4us = delay1024us >> 8;
	delay8us = delay1024us >> 7;
	delay16us = delay1024us >> 6;
	delay32us = delay1024us >> 5;
	delay64us = delay1024us >> 4;
	delay128us = delay1024us >> 3;
	delay200us = delay1024us * 200 / 1024;

	len = 0;
	addr = value = 0;
	while (1) {
		c = uart_getbyte();
		if (c >= 0) {
			if (len > 0) {
				buffer[256 - len] = c;
				len--;
			} else if (c < 0x80) {
				value = (value << 7) | c;
			} else {
				switch (c) {
				case OP_SETBINARY256:
					len = 256;
					break;
				case OP_SETADDRESS:
					addr = value;
					break;
				case OP_CHECKSUM:
					for (i = c = 0; i < 256; i++) {
						c += buffer[i];
					}
					uart_putbyte(c);
					break;
				case OP_PROGRAM:
					for (i = 0; i < 256; i += 128) {
						n = write_block((unsigned char *)addr + i, buffer + i);
						write((unsigned char *)addr + i, 128);
					}
					break;
				}
			}
		}
	}
}
